part of stagexl.engine;

class RenderFrameBuffer {

  RenderTexture _renderTexture = null;
  RenderStencilBuffer _renderStencilBuffer = null;
  RenderContextWebGL _renderContext = null;

  int _contextIdentifier = -1;
  gl.Framebuffer _framebuffer = null;
  gl.RenderingContext _renderingContext = null;

  RenderFrameBuffer.rawWebGL(int width, int height) {
    _renderTexture = new RenderTexture.rawWebGL(width, height);
    _renderStencilBuffer = new RenderStencilBuffer.rawWebGL(width, height);
  }

  //---------------------------------------------------------------------------

  int get width => _renderTexture.width;
  int get height => _renderTexture.height;
  int get contextIdentifier => _contextIdentifier;

  RenderTexture get renderTexture => _renderTexture;
  RenderStencilBuffer get renderStencilBuffer => _renderStencilBuffer;

  //---------------------------------------------------------------------------

  /// Call the dispose method to release memory allocated by WebGL.

  void dispose() {

    if (_renderTexture != null) _renderTexture.dispose();
    if (_renderStencilBuffer != null) _renderStencilBuffer.dispose();
    if (_framebuffer != null) _renderingContext.deleteFramebuffer(_framebuffer);

    _contextIdentifier = -1;
    _renderTexture = null;
    _renderStencilBuffer = null;
    _framebuffer = null;
  }

  //---------------------------------------------------------------------------

  void activate(RenderContextWebGL renderContext) {

    if (this.contextIdentifier != renderContext.contextIdentifier) {

      _renderContext = renderContext;
      _contextIdentifier = renderContext.contextIdentifier;
      _renderingContext = renderContext.rawContext;
      _framebuffer = renderContext.rawContext.createFramebuffer();
      _renderContext.activateRenderTexture(_renderTexture);
      _renderContext.activateRenderStencilBuffer(_renderStencilBuffer);

      var target = gl.FRAMEBUFFER;
      var color = gl.COLOR_ATTACHMENT0;
      var colorTarget = gl.TEXTURE_2D;
      var colorData = _renderTexture.texture;
      var stencil = gl.DEPTH_STENCIL_ATTACHMENT;
      var stencilTarget = gl.RENDERBUFFER;
      var stencilData = _renderStencilBuffer.renderbuffer;

      _renderingContext.bindFramebuffer(target, _framebuffer);
      _renderingContext.framebufferTexture2D(target, color, colorTarget, colorData, 0);
      _renderingContext.framebufferRenderbuffer(target, stencil, stencilTarget, stencilData);

    } else {

      _renderingContext.bindFramebuffer(gl.FRAMEBUFFER, _framebuffer);

    }
  }

}